# darwin.tcl
#
# Configuration feature definitions for the configuration 'darwin'
#
# Editing instructions begin at "START EDIT HERE" below.

set config [Oc_Config RunPlatform]

if {![string match [string tolower [file rootname [file tail [info script]]]] \
        [$config GetValue platform_name]]} {
    error "Configuration cache file '[info script]' 
sourced by '[$config GetValue platform_name]'"
}

########################################################################
# START EDIT HERE
# OOMMF software cannot classify your computing platform type.  See
# the Installation section of the OOMMF User Manual for instructions on 
# how to add a new platform type to the collection of types recognized 
# by OOMMF software.
#
# Say you add the new platform type 'foo' to describe your computing
# platform.  Then copy this file to ./foo.tcl , and edit it to
# describe your computing platform.  In its initial state, this file
# describes a rather generic Linux system, so there shouldn't be much
# editing required for any Unix-based platform.  Other platforms may
# be more difficult or impossible.  In any event, please e-mail the
# OOMMF developers for assistance setting up OOMMF for your particular
# circumstances.
########################################################################
# In order to properly build, install, and run on your computing 
# platform, the OOMMF software must know certain features of your
# computing environment.  In this file are lines which set the value of 
# certain features of your computing environment.  Each line looks like:
#
# $config SetValue <feature> {<value>}
#
# where each <feature> is the name of some feature of interest,
# and <value> is the value which is assigned to that feature in a
# description of your computing environment.  Your task is to edit
# the values as necessary to properly describe your computing 
# environment.
#
# The character '#' at the beginning of a line is a comment character.
# It causes the contents of that line to be ignored.  To select
# among lines providing alternative values for a feature, uncomment the
# line containing the proper value.
#
# The features in this file are divided into three sections.  The first
# section (REQUIRED CONFIGURATION) includes features which require you 
# to provide a value.  The second section (OPTIONAL CONFIGURATION)
# includes features which have usable default values, but which you
# may wish to customize.  The third section (ADVANCED CONFIGURATION)
# contains features which you probably do not need or want to change
# without a good reason.
########################################################################
# REQUIRED CONFIGURATION

########################################################################
# OPTIONAL CONFIGURATION

# Set the feature 'program_compiler_c++' to the program to run on this 
# platform to compile source code files written in the language C++ into 
# object files.  Select from the choices below.  If the compiler is not 
# in your path, be sure to use the whole pathname.  Also include any 
# options required to instruct your compiler to only compile, not link.  
#
# If your compiler is not listed below, additional features will
# have to be added in the ADVANCED CONFIGURATION section below to 
# describe to the OOMMF software how to operate your compiler.  Send
# e-mail to the OOMMF developers for assistance.
#
# The GNU C++ compiler 'g++'
# <URL:http://www.gnu.org/software/gcc/gcc.html>
# <URL:http://egcs.cygnus.com/>
$config SetValue program_compiler_c++ {g++ -c}

# Set the feature 'path_directory_temporary' to the name of an existing 
# directory on your computer in which OOMMF software should write 
# temporary files.  All OOMMF users must have write access to this 
# directory.
#
# $config SetValue path_directory_temporary {/tmp}

########################################################################
# SUPPORT PROCEDURES
#
# Routine to guess the processor type
proc GuessCPU {} {
   # Sample output from various system introspection facilities, on a
   # PowerBook G4:
   #   hwprefs cpu_type           --> 7455 v2.1
   #   machine                    --> ppc7450
   #   system_profiler SPHardwareDataType | grep 'Machine Name'
   #                              --> Machine Name: PowerBook G4
   #   system_profiler SPHardwareDataType | grep 'CPU Type'
   #                              --> CPU Type: PowerPC G4  (2.1)
   #   sysctl hw.model            --> hw.model: PowerBook3,4
   #   sysctl hw.cputype          --> hw.cputype: 18
   #   sysctl hw.cpusubtype       --> hw.cpusubtype: 11
   #   sysctl hw.optional.altivec --> hw.optional.altivec: 1
   # From a eMac:
   #   hwprefs cpu_type           --> <hwprefs not installed>
   #   machine                    --> ppc7450
   #   system_profiler SPHardwareDataType | grep 'Machine Name'
   #                              --> <empty string>
   #   system_profiler SPHardwareDataType | grep 'Machine Model'
   #                              --> Machine Model: dMac
   #   system_profiler SPHardwareDataType | grep 'CPU Type'
   #                              --> CPU Type: PowerPC G4  (2.1)
   #   sysctl hw.model            --> hw.model: PowerMac4,4
   #   sysctl hw.cputype          --> hw.cputype: 18
   #   sysctl hw.cpusubtype       --> hw.cpusubtype: 11
   #   uname -p                   --> powerpc
   #   echo $HOSTTYPE             --> powermac
   #   echo $MACHTYPE             --> powerpc
   #
   set guess unknown
   catch {
      set cpuinfo {}
      # Try, in order, machine, hwprefs, system_profiler, sysctl
      if {![catch {exec machine} cpustr]} {
         regexp {ppc([0-9]+)} $cpustr dummy cpuinfo
      }
      if {[string match {} $cpuinfo] && \
             ![catch {exec hwprefs cpu_type} cpustr]} {
         set cpuinfo [lindex $cpustr 0]
      }
      if {[string match {} $cpuinfo] && \
             ![catch {exec system_profiler SPHardwareDataType} cpustr]} {
         set cpustr [split $cpustr "\n"]
         set cpurec [lsearch -glob $cpustr "*CPU Type*"]
         if {$cpurec>=0} {
            regexp {(G[0-9]+)} [lindex $cpustr $cpurec] dummy cpuinfo
         }
      }
      if {[string match {} $cpuinfo] && \
             ![catch {exec sysctl hw.model} cpustr]} {
         # sysctl hw.model returns strings of the form
         #       hw.model: PowerBook3,4
         # Unfortunately, AFAICT there is no simple mapping from
         # these model numbers to processors.  Below is a good(?)
         # guess, as of Jan-2006.
         set cpustr [string trim $cpustr]
         regexp {^hw.model: *(.*)$} $cpustr dummy cpustr
         regexp {^([^0-9]*)([0-9]+),?([^,]*)} $cpustr dummy type verA verB
         if {[string match PowerMac $type]} {
            if {$verA==4 && $verB==1} {
               set cpuinfo G3
            } elseif {$verA==10} {
               set cpuinfo G4
            } elseif {$verA>=7} {
               set cpuinfo G5
            } elseif {$verA>=3} {
               set cpuinfo G4
            } else {
               set cpuinfo G3
            }
         } elseif {[string match PowerBook $type]} {
            if {$verA==4 || ($verA==3 && $verB==1)} {
               set cpuinfo G3
            } elseif {$verA>=3} {
               set cpuinfo G4
            } else {
               set cpuinfo G3
            }
         } elseif {[string match RackMac $type]} {
            if {$verA<3} {
               set cpuinfo G4
            } else {
               set cpuinfo G5
            }
         } elseif {[string match iMac $type]} {
            # Note: Supposedly only the early iMac's returned "iMac"
            # as their machine id; later models return "PowerMac"
            set cpuinfo G3
         }
      }
      if {[string match G* $cpuinfo]} {
         # Map G# info a 3 or 4 digit model number
         switch -exact $cpuinfo {
            G3 { set cpuinfo 750 }
            G4 { set cpuinfo 7400 }
            G5 { set cpuinfo 970 }
            default { set cpuinfo {} }
         }
      }
      if {![string match {} $cpuinfo]} {
         set guess $cpuinfo
      }
   }
   return $guess
}

# Routine to guess the gcc version
proc GuessGccVersion { gcc } {
    set guess {}
    catch {
	set fptr [open "|$gcc --version" r]
	set verstr [read $fptr]
	close $fptr
	regexp -- {[0-9][0-9.]+} $verstr guess
    }
    return [split $guess "."]
}

########################################################################
# ADVANCED CONFIGURATION

# Compile in thread support?
set oommf_threads [$config GetValue oommf_threads]

# Compiler option processing...
if {[string match g++ [file tail [lindex \
        [$config GetValue program_compiler_c++] 0]]]} {
    # ...for GNU g++ C++ compiler
    set gccversion [GuessGccVersion \
			[lindex [$config GetValue program_compiler_c++] 0]]

    set opts {}
    # set opts [list -O0]     ;# No optimization
    # set opts [list -O%s]    ;# Standard optimization
    if {[string match {} $opts] && $gccversion>=3.3} {
	# Processor specific optimization
	set cputype [GuessCPU]
	if {[string match 7?? $cputype]} {
	    # G3; use default opts string
	} elseif {[string match 9?? $cputype]} {
	    # G5
	    set opts [list -fast]
	} elseif {[string match 74?? $cputype]} {
	    # G4
	    if {$cputype>=7450} {
		set opts [list -fast -mcpu=7450]
	    } else {
		set opts [list -fast -mcpu=7400]
	    }
	}
    }
    if {[string match {} $opts]} {
	set opts [list -O2 -ffast-math -fstrict-aliasing]
    }

    # Compile in thread support?
    if {$oommf_threads} {
	lappend opts -DTCL_THREADS -DOOMMF_THREADS=1
    } else {
	lappend opts -DOOMMF_THREADS=0
    }

    $config SetValue program_compiler_c++_option_opt "format \"$opts\""

    $config SetValue program_compiler_c++_option_out {format "-o \"%s\""}
    $config SetValue program_compiler_c++_option_src {format \"%s\"}
    $config SetValue program_compiler_c++_option_inc {format "\"-I%s\""}
    $config SetValue program_compiler_c++_option_def {format "\"-D%s\""}

    # Widest natively support floating point type
    $config SetValue program_compiler_c++_typedef_realwide "double"

    # Directories to exclude from explicit include search path, i.e.,
    # the -I list.  Some versions of gcc complain if "system" directories
    # appear in the -I list.
    $config SetValue \
    	program_compiler_c++_system_include_path [list /usr/include]
}

# The program to run on this platform to link together object files and
# library files to create an executable binary.
#
# Use the selected compiler to control the linking.
$config SetValue program_linker [lindex \
        [$config GetValue program_compiler_c++] 0]

# Linker option processing...
if {[string match g++ [file tail [lindex \
        [$config GetValue program_linker] 0]]]} {
    # ...for GNU g++ as linker
    $config SetValue program_linker_option_obj {format \"%s\"}
    $config SetValue program_linker_option_out {format "-o \"%s\""}
    $config SetValue program_linker_option_lib {format \"%s\"}
    $config SetValue program_linker_uses_-L-l {1}
}

# The program to run on this platform to create a single library file out
# of many object files.
$config SetValue program_libmaker {ar cr}

if {[string match ar [file tail [lindex \
        [$config GetValue program_libmaker] 0]]]} {
    # Option processing for ar
    $config SetValue program_libmaker_option_obj {format \"%s\"}
    $config SetValue program_libmaker_option_out {format \"%s\"}
}

# The absolute, native filename of the null device
$config SetValue path_device_null {/dev/null}

# A partial Tcl command (or script) which when completed by lappending
# a file name stem and evaluated returns the corresponding file name for
# an executable on this platform
$config SetValue script_filename_executable {format %s}

# A partial Tcl command (or script) which when completed by lappending
# a file name stem and evaluated returns the corresponding file name for
# an object file on this platform
$config SetValue script_filename_object {format %s.o}

# A partial Tcl command (or script) which when completed by lappending
# a file name stem and evaluated returns the corresponding file name for
# a static library on this platform
$config SetValue script_filename_static_library {format lib%s.a}

########################################################################
unset config
