# FILE: lintel.tcl
#
# Configuration feature definitions for the configuration 'lintel'
#
# Editing instructions begin at "START EDIT HERE" below.

set config [Oc_Config RunPlatform]

if {![string match [string tolower [file rootname [file tail [info script]]]] \
        [$config GetValue platform_name]]} {
    error "Configuration cache file '[info script]' 
sourced by '[$config GetValue platform_name]'"
}

########################################################################
# START EDIT HERE
# In order to properly build, install, and run on your computing 
# platform, the OOMMF software must know certain features of your
# computing environment.  In this file are lines which set the value of 
# certain features of your computing environment.  Each line looks like:
#
# $config SetValue <feature> {<value>}
#
# where each <feature> is the name of some feature of interest,
# and <value> is the value which is assigned to that feature in a
# description of your computing environment.  Your task is to edit
# the values as necessary to properly describe your computing 
# environment.
#
# The character '#' at the beginning of a line is a comment character.
# It causes the contents of that line to be ignored.  To select
# among lines providing alternative values for a feature, uncomment the
# line containing the proper value.
#
# The features in this file are divided into three sections.  The first
# section (REQUIRED CONFIGURATION) includes features which require you 
# to provide a value.  The second section (OPTIONAL CONFIGURATION)
# includes features which have usable default values, but which you
# may wish to customize.  The third section (ADVANCED CONFIGURATION)
# contains features which you probably do not need or want to change
# without a good reason.
########################################################################
# REQUIRED CONFIGURATION

# Set the feature 'program_compiler_c++' to the program to run on this 
# platform to compile source code files written in the language C++ into 
# object files.  Select from the choices below.  If the compiler is not 
# in your path, be sure to use the whole pathname.  Also include any 
# options required to instruct your compiler to only compile, not link.  
#
# If your compiler is not listed below, additional features will
# have to be added in the ADVANCED CONFIGURATION section below to 
# describe to the OOMMF software how to operate your compiler.  Send
# e-mail to the OOMMF developers for assistance.
#
# The GNU C++ compiler 'g++'
# <URL:http://www.gnu.org/software/gcc/gcc.html>
# <URL:http://egcs.cygnus.com/>
$config SetValue program_compiler_c++ {g++ -c}
#
# The Portland Group 'pgCC'
# <URL:http://www.pgroup.com/>
#$config SetValue program_compiler_c++ {pgCC -c}
#
# Temporary kludge to get pgCC to play with pimake.  First, create
# a shell script with the contents:
#
#      #!/bin/sh
#      # Arguments: CFLAGS library objs
#      flags="$1"
#      shift
#      library="$1"
#      shift
#      objs="$*"
#      pgCC --prelink_objects $flags $objs
#      ar cr $library lintel/*.o
#
# Call this pgcclib.sh and place it in the PATH.  Then, edit this
# file to include
#
#         --one_instantiation_per_object
#
# in the "opts" variable of the pgCC compiler section below, and
# exclude
#
#         -Minline
#
# (all flavors).  Then cd into the oommf/pkg directory, and run
# pimake from there, to build all the libraries.  Then re-edit
# this file to remove --one_instantiation_per_object from opts,
# and re-include -Minline (if desired).  Then cd back to the
# oommf top level directory, and re-run pimake to complete
# the build process.
#
#
# The Intel C++ compiler 'icpc'
# <URL:http://www.intel.com>
#$config SetValue program_compiler_c++ {icpc -c}

########################################################################
# OPTIONAL CONFIGURATION
#
# Set the feature 'path_directory_temporary' to the name of an existing 
# directory on your computer in which OOMMF software should write 
# temporary files.  All OOMMF users must have write access to this 
# directory.
#
# $config SetValue path_directory_temporary {/tmp}

########################################################################
# SUPPORT PROCEDURES
#
# Routine to guess the CPU using /procs/cpuinfo provided by Linux kernel
proc GuessCPU {} {
   set guess unknown
   catch {
      set fptr [open /proc/cpuinfo r]
      set cpustr [read $fptr]
      close $fptr
      # Look at first processor info only
      set tmplst [split $cpustr "\n"]
      set record_end [lsearch $tmplst {}]
      if {$record_end>0} {
         set tmplst [lreplace $tmplst $record_end end]
      }
      set cpustr [join $tmplst ":"]
      foreach {x y} [split $cpustr ":"] {
         set cpuinfo([string trim $x]) [string tolower [string trim $y]]
      }
      # Try to guess processor based on Intel family + model info
      # Note: The /proc/cpuinfo "flags" values can be decoded with
      # reference to /usr/include/asm/cpufeature.h.
      if {[string match "*intel*" $cpuinfo(vendor_id)]} {
         if {$cpuinfo(cpu family) >= 15} {
            set guess pentium4
	    if {$cpu_model == 3} {
	       set guess prescott
	    } elseif {$cpu_model>=4} {
              set guess nocona
            }
         } elseif {$cpuinfo(cpu family)==6} {
            if {$cpuinfo(model)<3} {
               set guess pentiumpro
            } elseif {$cpuinfo(model)<7} {
               set guess pentium2
            } elseif {$cpu_model == 9 || $cpu_model == 13} {
                set guess pentium-m
            } else {
               set guess pentium3
            }
         }
      }
      if {[string match "unknown" $guess]} {
         # Otherwise, try to match against "model name"
         switch -regexp -- $cpuinfo(model name) {
            {^386}                    { set guess 386 }
            {^486}                    { set guess 486 }
            {pentium(\(r\)|) (iv|4)}  { set guess pentium4 }
            {pentium(\(r\)|) (iii|3)} { set guess pentium3 }
            {pentium(\(r\)|) (ii|2)}  { set guess pentium2 }
            {pentium(\(r\)|) pro}     { set guess pentiumpro }
            {pentium}                 { set guess pentium }
            {overdrive podp5v83}      { set guess pentium }
            {opteron}                 { set guess opteron }
            {athlon(\(tm\)|) 64}      { set guess athlon64 }
            {athlon(\(tm\)|) fx}      { set guess athlon-fx }
            {athlon(\(tm\)|) mp}      { set guess athlon-mp }
            {athlon(\(tm\)|) xp}      { set guess athlon-xp }
            {athlon(\(tm\)|) (iv|4)}  { set guess athlon-4 }
            {athlon(\(tm\)|) (tbird)} { set guess athlon-tbird }
            {athlon}                  { set guess athlon }
            {^k8}                     { set guess k8 }
            {^k6-3}                   { set guess k6-3 }
            {^k6-2}                   { set guess k6-2 }
            {^k6}                     { set guess k6 }
            {^k5}                     { set guess k5 }
         }
      }
   }
   return $guess
}

# Routine to guess the gcc version
proc GuessGccVersion { gcc } {
    set guess {}
    catch {
	set fptr [open "|$gcc --version" r]
	set verstr [read $fptr]
	close $fptr
	regexp -- {[0-9][0-9.]+} $verstr guess
    }
    return [split $guess "."]
}

########################################################################
# ADVANCED CONFIGURATION

# If compiler is specified in the oommf/config/local/options.tcl file,
# then that value overrides selection above.  The override setting is
# set by an entry like this, in the oommf/config/local/options.tcl file:
#   Oc_Option Add * Oc_Config compiler_override {icpc -c}
if {[catch {$config GetValue program_compiler_c++_override} compiler] == 0} {
    $config SetValue program_compiler_c++ $compiler
}

# Compile in thread support?
set oommf_threads [$config GetValue oommf_threads]

# Compiler option processing...
set ccbasename [file tail [lindex [$config GetValue program_compiler_c++] 0]]
if {[string match g++ $ccbasename]} {
    # ...for GNU g++ C++ compiler
    set gccversion [GuessGccVersion \
			[lindex [$config GetValue program_compiler_c++] 0]]

    # Optimization options
    # set opts [list -O0 -fno-inline -ffloat-store]  ;# No optimization
    # set opts [list -O%s]                      ;# Minimal optimization
    set opts [list -O2 -ffast-math -fstrict-aliasing]
    # -O3 can be tried instead of -O2, but -O3 is buggy in some
    # releases of gcc. You can also try adding -malign-double.
    # See next block for comments concerning -fomit-frame-pointer.
    # Processor architecture selection is farther down.

    # Frame pointer: Some versions of gcc don't handle exceptions
    # properly w/o frame-pointers.  This typically manifests as
    # Oxs dying without an error message while loading a MIF file.
    # Interestingly, including -momit-leaf-frame-pointer appears
    # to work around this problem, at least on some systems.  YMMV;
    # Comment this out if the aforementioned problem occurs.
    lappend opts -fomit-frame-pointer -momit-leaf-frame-pointer

    # Compile in thread support?
    if {$oommf_threads} {
       lappend opts -DTCL_THREADS -DOOMMF_THREADS=1
    } else {
       lappend opts -DOOMMF_THREADS=0
    }

    # The next block sets cpu-specific options, based on a guess (see
    # GuessCPU proc defined at the top of this file) of the current
    # CPU and gcc version.  If this guess is wrong, or you want to
    # build for a generic CPU, unset cpuopts below.  NOTE: This is a
    # first try, so it is most certainly broken!
    set cputype [GuessCPU]
    if {[llength $gccversion]>1} {
       # Set cpu-dependent compiler options
       set verA [lindex $gccversion 0]
       set verB [lindex $gccversion 1]
       if {$verA==2 && $verB>=95} {
          switch -glob -- $cputype {
             k6*      { set cpuopts [list -march=k6] }
             pentium  { set cpuopts [list -march=pentium] }
             athlon*  -
             opteron  -
             prescott -
             nocona   -
             pentium* { set cpuopts [list -march=pentiumpro] }
          }
       } elseif {$verA==3 && $verB<1} {
          switch -glob -- $cputype {
             pentium  { set cpuopts [list -march=pentium] }
             prescott -
             nocona   -
             pentium* { set cpuopts [list -march=pentiumpro] }
             k6*      { set cpuopts [list -march=k6] }
             opteron  -
             athlon*  { set cpuopts [list -march=athlon] }
          }
       } elseif {$verA>=3} {
          switch -exact -- $cputype {
             pentium    { set cpuopts [list -march=pentium] }
             pentiumpro { set cpuopts [list -march=pentiumpro] }
             pentium2   { set cpuopts [list -march=pentium2] }
             pentium3   { set cpuopts [list -march=pentium3] }
	     pentium-m  { set cpuopts \
			     [list -march=pentium-m -msse2 -mfpmath=sse] }
             pentium4   { set cpuopts \
                             [list -march=pentium4 -msse2 -mfpmath=sse] }
	     prescott   { set cpuopts \
                             [list -march=prescott -msse3 -mfpmath=sse] }
             nocona     { set cpuopts \
                             [list -march=nocona -msse3 -mfpmath=sse] }
             k6         { set cpuopts [list -march=k6] }
             k6-2       { set cpuopts [list -march=k6-2] }
             k6-3       { set cpuopts [list -march=k6-3] }
             athlon     { set cpuopts [list -march=athlon] }
             athlon-tbird { set cpuopts [list -march=athlon-tbird] }
             athlon-4   { set cpuopts [list -march=athlon-4] }
             athlon-xp  { set cpuopts [list -march=athlon-xp] }
             athlon-mp  { set cpuopts [list -march=athlon-mp] }
             athlon-fx  -
             athlon64   -
             opteron    -
             k8         { set cpuopts \
                             [list -march=k8 -m32 -msse2 -mfpmath=sse] }
          }
       }
    }
    # You can override the GuessCPU results by directly setting or
    # unsetting the cpuopts variable, e.g.,
    #
    #    unset cpuopts
    #    set cpuopts [list -march=athlon]
    #
    if {[info exists cpuopts] && [llength $cpuopts]>0} {
       set opts [concat $opts $cpuopts]
    }
    $config SetValue program_compiler_c++_option_opt "format \"$opts\""
    # NOTE: If you want good performance, be sure to edit ../options.tcl
    #  or ../local/options.tcl to include the line
    #    Oc_Option Add * Platform cflags {-def NDEBUG}
    #  so that the NDEBUG symbol is defined during compile.
    $config SetValue program_compiler_c++_option_out {format "-o \"%s\""}
    $config SetValue program_compiler_c++_option_src {format \"%s\"}
    $config SetValue program_compiler_c++_option_inc {format "\"-I%s\""}
 
    # Compiler warnings:
    # Omitted: -Wredundant-decls -Wshadow -Wcast-align
    # I would also like to use -Wcast-qual, but casting away const is
    # needed on some occasions to provide "conceptual const" functions in
    # place of "bitwise const"; cf. p76-78 of Meyer's book, "Effective C++."
    #
    # NOTE: -Wno-uninitialized is required after -Wall by gcc 2.8+ because
    # of an apparent bug.  -Winline is out because of failures in the STL.
    # Depending on the gcc version, the following options may also be
    # available:     -Wbad-function-cast     -Wstrict-prototypes
    #                -Wmissing-declarations  -Wnested-externs
    $config SetValue program_compiler_c++_option_warn {format "-Wall \
        -W -Wpointer-arith -Wwrite-strings -Wmissing-prototypes \
        -Woverloaded-virtual -Wsynth -Werror"}
    $config SetValue program_compiler_c++_option_debug {format "-g"}
    $config SetValue program_compiler_c++_option_def {format "\"-D%s\""}

    # Wide floating point type.
    # NOTE: On the Linux/x86+gcc platform, "long double" provides
    # somewhat better precision than "double", but at a cost of
    # increased memory usage and a decrease in speed.  (At this writing,
    # long double takes 12 bytes of storage as opposed to 8 for double,
    # but provides the x86 native floating point format having approx.
    # 19 decimal digits precision as opposed to 16 for double.)
    # Default is "double".
    $config SetValue program_compiler_c++_typedef_realwide "double"

    # Directories to exclude from explicit include search path, i.e.,
    # the -I list.  Some versions of gcc complain if "system" directories
    # appear in the -I list.
    $config SetValue \
    	program_compiler_c++_system_include_path [list /usr/include]

    # Other compiler properties
    $config SetValue \
            program_compiler_c++_property_optimization_breaks_varargs 0
} elseif {[string match pgCC $ccbasename]} {
    # ...for Portland Group C++ compiler
    # Optimization options
    # set opts [list -O0]  ;# No optimization
    # set opts [list -O%s] ;# Minimal
    set opts [list -fast -Mlre=assoc \
	    -Minline=levels:10 \
            -Mvect=noaltcode,assoc,idiom,recog,nosse,notransform \
	    --instantiation_dir lintel]
    #set opts [list -fast -Minline=levels:10,lib:lintel \
    #    --instantiation_dir lintel -Mvect -Mcache_align]
    # Some suggested options: -fast, -Minline=levels:10,
    # -O3, -Mipa, -Mcache_align, -Mvect, -Mvect=sse, -Mconcur.
    #    WRT -Mvect, the transform suboption produces segfault upon
    # execution.  Use notransform to explicitly disable.
    #    If you use -Mconcur, be sure to include -Mconcur on the
    # link line as well, and set the environment variable NCPUS
    # to the number of CPU's to use at execution time.
    #    If you use -Mipa, include it also on the link
    # line.  Using IPA info requires two build passes.  After
    # building once, wipe away the object modules with
    # 'tclsh oommf.tcl pimake objclean', and then rebuild.
    # In theory, anyway.  I haven't been able to get this
    # switch to work with pgCC 5.0-2.
    #
    # The -fast option sets processor specific options.  You can
    # override these by directly setting the cpuopts variable, e.g.,
    #
    # set cpuopts [list -tp k8-32]
    #
    # See the compiler documentation for options to the -tp
    # flag, but possibilities include p5, p6, p7, k8-32,
    # k8-64, and px.  The last is generic x86.
    if {[info exists cpuopts] && [llength $cpuopts]>0} {
	set opts [concat $opts $cpuopts]
    }
    #
    # Template handling for libraries
    # lappend opts --one_instantiation_per_object

    # Compile in thread support?
    if {$oommf_threads} {
       lappend opts -DTCL_THREADS -DOOMMF_THREADS=1
    } else {
       lappend opts -DOOMMF_THREADS=0
    }

    $config SetValue program_compiler_c++_option_opt "format \"$opts\""

    # NOTE: If you want good performance, be sure to edit ../options.tcl
    #  or ../local/options.tcl to include the line
    #    Oc_Option Add * Platform cflags {-def NDEBUG}
    #  so that the NDEBUG symbol is defined during compile.
    $config SetValue program_compiler_c++_option_out {format "-o \"%s\""}
    $config SetValue program_compiler_c++_option_src {format \"%s\"}
    $config SetValue program_compiler_c++_option_inc {format "\"-I%s\""}
 
    # Compiler warnings:
    $config SetValue program_compiler_c++_option_warn {format \
	    "-Minformlevel=warn"}
    $config SetValue program_compiler_c++_option_debug {format "-g"}
    $config SetValue program_compiler_c++_option_def {format "\"-D%s\""}

    # Wide floating point type.
    $config SetValue program_compiler_c++_typedef_realwide "double"

    # Directories to exclude from explicit include search path, i.e.,
    # the -I list.  Some of the gcc versions don't play well with
    # the Portland Group compilers, so keep them off the compile line.
    $config SetValue \
    	program_compiler_c++_system_include_path [list /usr/include]

    # Other compiler properties
    $config SetValue \
            program_compiler_c++_property_optimization_breaks_varargs 0
} elseif {[string match icpc $ccbasename]} {
    # ...for Intel's icpc C++ compiler
    # set opts [list -O0]
    # set opts [list -O%s]
    set opts [list -fast -ansi_alias -wd1572]
    # NOTES on program_compiler_c++_option_opt:
    #   If you use -ipo, or any other flag that enables interprocedural
    #     optimizations (IPO) such as -fast, then the program_linker
    #     value (see below) needs to have -ipo added.
    #   If you add -parallel to program_compiler_c++_option_opt, then
    #     add -parallel to the program_linker value too.
    #   For good performance, be sure that ../options.tcl
    #     or ../local/options.tcl includes the line
    #         Oc_Option Add * Platform cflags {-def NDEBUG}
    #     so that the NDEBUG symbol is defined during compile.
    #   The -wd1572 option disables warnings about floating
    #     point comparisons being unreliable.

    # Compile in thread support?
    if {$oommf_threads} {
       lappend opts -DTCL_THREADS -DOOMMF_THREADS=1
    } else {
       lappend opts -DOOMMF_THREADS=0
    }

    $config SetValue program_compiler_c++_option_opt "format \"$opts\""

    $config SetValue program_compiler_c++_option_out {format "-o \"%s\""}
    $config SetValue program_compiler_c++_option_src {format \"%s\"}
    $config SetValue program_compiler_c++_option_inc {format "\"-I%s\""}
    $config SetValue program_compiler_c++_option_def {format "\"-D%s\""}
    $config SetValue program_compiler_c++_option_debug {format "-g"}

    $config SetValue program_compiler_c++_option_warn \
        { format "-Wall -Werror -wd1418,1419,279,810,981,383,1572" }
    # { format "-w0 -verbose \
    #   -msg_disable undpreid,novtbtritmp,boolexprconst,badmulchrcom" }

    # Wide floating point type.
    $config SetValue program_compiler_c++_typedef_realwide "double"
}


# The program to run on this platform to link together object files and
# library files to create an executable binary.
set lbasename $ccbasename
if {[string match g++ $lbasename]} {
    # ...for GNU g++ as linker
    $config SetValue program_linker [list $lbasename]
    $config SetValue program_linker_option_obj {format \"%s\"}
    $config SetValue program_linker_option_out {format "-o \"%s\""}
    $config SetValue program_linker_option_lib {format \"%s\"}
    $config SetValue program_linker_rpath {format "-Wl,-rpath=%s"}
    $config SetValue program_linker_uses_-L-l {1}
} elseif {[string match pgCC $lbasename]} {
    # ...for Portland Group pgCC as linker
    $config SetValue program_linker [list $lbasename]
    if {[info exists opts]} {
	$config SetValue program_linker [concat [list $lbasename] $opts]
    }
    $config SetValue program_linker_option_obj {format \"%s\"}
    $config SetValue program_linker_option_out {format "-o \"%s\""}
    $config SetValue program_linker_option_lib {format \"%s\"}
    $config SetValue program_linker_uses_-L-l {1}
    $config SetValue program_linker_uses_-I-L-l {0}
} elseif {[string match icpc $lbasename]} {
    # ...for Intel's icpc as linker
    set linkcmdline [list $lbasename]
    # If -fast or other flags that enable interprocedural optimizations
    # (IPO) appear in the program_compiler_c++_option_opt value above,
    # then append those flags into program_linker too.
    if {[lsearch -exact $opts -fast]>=0} {
       lappend linkcmdline -ipo -lsvml
       # The svml library is needed for the dvl/spectrum executable
       # when compiled with some releases of the icpc 9.0 compiler.
    }
    if {[lsearch -exact $opts -parallel]>=0} {
       lappend linkcmdline -parallel
    }
    $config SetValue program_linker $linkcmdline
    unset linkcmdline

    $config SetValue program_linker_option_obj {format \"%s\"}
    $config SetValue program_linker_option_out {format "-o \"%s\""}
    $config SetValue program_linker_option_lib {format \"%s\"}
    $config SetValue program_linker_rpath {format "-Qoption,ld,-rpath=%s"}
    $config SetValue program_linker_uses_-L-l {1}
}
unset lbasename

# The program to run on this platform to create a single library file out
# of many object files.
if {[string match icpc $ccbasename]} {
    $config SetValue program_libmaker {xiar csr}
    $config SetValue program_libmaker_option_obj {format \"%s\"}
    $config SetValue program_libmaker_option_out {format \"%s\"}
} elseif {[string match pgCC $ccbasename]} {
    $config SetValue program_libmaker [subst {pgcclib.sh "$opts"}]
    $config SetValue program_libmaker_option_obj {format \"%s\"}
    $config SetValue program_libmaker_option_out {format \"%s\"}
} else {
    $config SetValue program_libmaker {ar cr}
    $config SetValue program_libmaker_option_obj {format \"%s\"}
    $config SetValue program_libmaker_option_out {format \"%s\"}
}
unset ccbasename

# The absolute, native filename of the null device
$config SetValue path_device_null {/dev/null}

# A partial Tcl command (or script) which when completed by lappending
# a file name stem and evaluated returns the corresponding file name for
# an executable on this platform
$config SetValue script_filename_executable {format %s}

# A partial Tcl command (or script) which when completed by lappending
# a file name stem and evaluated returns the corresponding file name for
# an object file on this platform
$config SetValue script_filename_object {format %s.o}

# A partial Tcl command (or script) which when completed by lappending
# a file name stem and evaluated returns the corresponding file name for
# a static library on this platform
$config SetValue script_filename_static_library {format lib%s.a}

########################################################################
unset config
