/* FILE: xpfloat.cc                    -*-Mode: c++-*-
 *
 * Class for extended floating point precision using
 * compensated summation.
 * 
 * NOTICE: Please see the file ../../LICENSE
 *
 * Last modified on: $Date: 2005/02/03 22:07:52 $
 * Last modified by: $Author: donahue $
 */

#include "xpfloat.h"

/* End includes */     

void Nb_Xpfloat::Accum(REALWIDE y)
{
  volatile REALWIDE sum1; // Mark as volatile to protect against
  volatile REALWIDE sum2; // problems arising from extra precision.
  volatile REALWIDE corrtemp;

  // Calculate sum
  sum1=y+corr;
  sum2=x+sum1;

  // Determine new correction term.  Do in two steps, as opposed to
  // "corrtemp = (x-sum2) + sum1", because a) some compilers ignore
  // parentheses (with regards to ordering) at high optimization levels,
  // and b) we want to be sure to drop any extra precision.
  corrtemp = x - sum2;
  corrtemp += sum1;

  // Store results
  corr = corrtemp;
  x=sum2;
}

void Nb_Xpfloat::Accum(const Nb_Xpfloat &y)
{
  if((x>0 && y.x>0) || (x<0 && y.x<0)) {
    Accum(y.corr);  Accum(y.x);
  } else {
    Accum(y.x);  Accum(y.corr);
  }
}

Nb_Xpfloat& Nb_Xpfloat::operator-=(const Nb_Xpfloat &y)
{
  if((x>0 && y.x<0) || (x<0 && y.x>0)) {
    Accum(-y.corr);  Accum(-y.x);
  } else {
    Accum(-y.x);  Accum(-y.corr);
  }
  return *this;
}

Nb_Xpfloat& Nb_Xpfloat::operator*=(REALWIDE y)
{
  x*=y;
  corr*=y;
  return *this;
}

Nb_Xpfloat operator+(const Nb_Xpfloat& x,const Nb_Xpfloat& y)
{
  Nb_Xpfloat z(x);
  z.Accum(y);
  return z;
}

Nb_Xpfloat operator-(const Nb_Xpfloat& x,const Nb_Xpfloat& y)
{
  Nb_Xpfloat z(y);
  z *= -1.;
  z.Accum(x);
  return z;
}

Nb_Xpfloat operator*(const Nb_Xpfloat& x,REALWIDE y)
{
  Nb_Xpfloat z(x);
  z*=y;
  return z;
}

Nb_Xpfloat& Nb_Xpfloat::operator*=(const Nb_Xpfloat& y)
{
  Nb_Xpfloat z(corr*y.corr);
  z.Accum(x*y.corr);
  z.Accum(y.x*corr);
  z.Accum(x*y.x);

  x = z.x;
  corr = z.corr;

  return *this;
}

Nb_Xpfloat operator*(const Nb_Xpfloat& x,const Nb_Xpfloat& y)
{
  Nb_Xpfloat z(x.corr*y.corr);
  z.Accum(x.x*y.corr);
  z.Accum(y.x*x.corr);
  z.Accum(x.x*y.x);
  return z;
}
